@@ -55,6 +55,7 @@ public:
     virtual ~ChartModel();
 
     Type type() const;
+    QString typeString() const;
 
     QVariant headerData(int section, Qt::Orientation orientation = Qt::Horizontal,
                         int role = Qt::DisplayRole) const override;
@@ -62,6 +63,12 @@ public:
     int rowCount(const QModelIndex& parent = QModelIndex()) const override;
     int columnCount(const QModelIndex& parent = QModelIndex()) const override;
 
+    int maximumDatasetCount() const
+    {
+        return m_maxDatasetCount;
+    }
+    void setMaximumDatasetCount(int count);
+
     qint64 totalCostAt(qint64 timeStamp) const;
 
 public slots:
@@ -69,12 +76,15 @@ public slots:
     void clearData();
 
 private:
+    void resetColors();
+
     ChartData m_data;
     Type m_type;
     // we cache the pens and brushes as constructing them requires allocations
     // otherwise
     QVector<QPen> m_columnDataSetPens;
     QVector<QBrush> m_columnDataSetBrushes;
+    int m_maxDatasetCount;
 };
 
 #endif // CHARTMODEL_H
