@@ -7,10 +7,17 @@
 #include "chartwidget.h"
 
 #include <QApplication>
+#include <QCheckBox>
+#include <QFileDialog>
+#include <QLabel>
 #include <QMenu>
 #include <QPainter>
+#include <QPushButton>
 #include <QRubberBand>
+#include <QSpinBox>
+#include <QSvgGenerator>
 #include <QTextStream>
+#include <QToolBar>
 #include <QToolTip>
 #include <QVBoxLayout>
 
@@ -24,6 +31,7 @@
 #include <KChartGridAttributes>
 #include <KChartHeaderFooter>
 #include <KChartLegend>
+#include <KMessageBox>
 
 #include <KColorScheme>
 #include <KLocalizedString>
@@ -38,6 +46,33 @@
 using namespace KChart;
 
 namespace {
+KChart::TextAttributes fixupTextAttributes(KChart::TextAttributes attributes, const QPen& foreground, float pointSize)
+{
+    attributes.setPen(foreground);
+    auto fontSize = attributes.fontSize();
+    fontSize.setAbsoluteValue(pointSize);
+    attributes.setFontSize(fontSize);
+    return attributes;
+}
+
+QPointF localPos(QMouseEvent* event)
+{
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+    return event->localPos();
+#else
+    return event->position();
+#endif
+}
+
+QPoint globalPos(QMouseEvent* event)
+{
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+    return event->globalPos();
+#else
+    return event->globalPosition().toPoint();
+#endif
+}
+
 class TimeAxis : public CartesianAxis
 {
     Q_OBJECT
@@ -119,9 +154,57 @@ private:
 ChartWidget::ChartWidget(QWidget* parent)
     : QWidget(parent)
     , m_chart(new Chart(this))
+    , m_legend(new Legend(m_chart))
     , m_rubberBand(new ChartRubberBand(this))
 {
+    auto m_chartToolBar = new QToolBar(this);
+
+    auto m_exportAsButton = new QPushButton(i18n("Export As..."), this);
+    connect(m_exportAsButton, &QPushButton::released, this, &ChartWidget::saveAs);
+
+    auto m_showLegend = new QCheckBox(i18n("Show legend"), this);
+    m_showLegend->setChecked(false);
+    connect(m_showLegend, &QCheckBox::toggled, this, [=](bool show) {
+        m_legend->setVisible(show);
+        m_chart->update();
+    });
+
+    auto m_showTotal = new QCheckBox(i18n("Show total cost graph"), this);
+    m_showTotal->setChecked(true);
+    connect(m_showTotal, &QCheckBox::toggled, this, [=](bool show) {
+        m_totalPlotter->setHidden(!show);
+        m_chart->update();
+    });
+    m_legend->setVisible(m_showLegend->checkState());
+
+    auto m_showDetailed = new QCheckBox(i18n("Show detailed cost graph"), this);
+    m_showDetailed->setChecked(true);
+    connect(m_showDetailed, &QCheckBox::toggled, this, [=](bool show) {
+        m_detailedPlotter->setHidden(!show);
+        m_chart->update();
+    });
+
+    auto stackedLabel = new QLabel(i18n("Stacked diagrams:"));
+    m_stackedDiagrams = new QSpinBox(this);
+    m_stackedDiagrams->setMinimum(0);
+    m_stackedDiagrams->setMaximum(50);
+    connect(m_stackedDiagrams, qOverload<int>(&QSpinBox::valueChanged), this,
+            [=](int value) { m_model->setMaximumDatasetCount(value + 1); });
+
+    m_chartToolBar->addWidget(m_exportAsButton);
+    m_chartToolBar->addSeparator();
+    m_chartToolBar->addWidget(m_showLegend);
+    m_chartToolBar->addSeparator();
+    m_chartToolBar->addWidget(m_showTotal);
+    m_chartToolBar->addWidget(m_showDetailed);
+    m_chartToolBar->addSeparator();
+    m_chartToolBar->addWidget(stackedLabel);
+    m_chartToolBar->addWidget(m_stackedDiagrams);
+
     auto layout = new QVBoxLayout(this);
+    layout->setContentsMargins(0, 0, 0, 0);
+    layout->setSpacing(0);
+    layout->addWidget(m_chartToolBar);
     layout->addWidget(m_chart);
     setLayout(layout);
 
@@ -186,7 +269,8 @@ void ChartWidget::setModel(ChartModel* model, bool minimalMode)
 
     auto* coordinatePlane = dynamic_cast<CartesianCoordinatePlane*>(m_chart->coordinatePlane());
     Q_ASSERT(coordinatePlane);
-    foreach (auto diagram, coordinatePlane->diagrams()) {
+    const auto diagrams = coordinatePlane->diagrams();
+    for (auto diagram : diagrams) {
         coordinatePlane->takeDiagram(diagram);
         delete diagram;
     }
@@ -197,59 +281,127 @@ void ChartWidget::setModel(ChartModel* model, bool minimalMode)
         coordinatePlane->setGlobalGridAttributes(grid);
     }
 
+    KColorScheme scheme(QPalette::Active, KColorScheme::Window);
+    QPen foreground(scheme.foreground().color());
+
     {
-        auto totalPlotter = new Plotter(this);
-        totalPlotter->setAntiAliasing(true);
+        KChart::GridAttributes grid = coordinatePlane->gridAttributes(Qt::Horizontal);
+        // Do not align view on main grid line, stretch grid to match datasets
+        grid.setAdjustBoundsToGrid(false, false);
+        coordinatePlane->setGridAttributes(Qt::Horizontal, grid);
+
+        m_legend->setOrientation(Qt::Vertical);
+        m_legend->setTitleText(QString());
+        m_legend->setSortOrder(Qt::DescendingOrder);
+
+        RelativePosition relPos;
+        relPos.setReferenceArea(coordinatePlane);
+        relPos.setReferencePosition(Position::NorthWest);
+        relPos.setAlignment(Qt::AlignTop | Qt::AlignLeft | Qt::AlignAbsolute);
+        relPos.setHorizontalPadding(Measure(3.0, KChartEnums::MeasureCalculationModeAbsolute));
+        relPos.setVerticalPadding(Measure(3.0, KChartEnums::MeasureCalculationModeAbsolute));
+
+        m_legend->setFloatingPosition(relPos);
+        m_legend->setTextAlignment(Qt::AlignLeft | Qt::AlignAbsolute);
+
+        m_chart->addLegend(m_legend);
+
+        BackgroundAttributes bkgAtt = m_legend->backgroundAttributes();
+        QColor background = scheme.background(KColorScheme::AlternateBackground).color();
+        background.setAlpha(200);
+        bkgAtt.setBrush(QBrush(background));
+        bkgAtt.setVisible(true);
+
+        TextAttributes textAttr = fixupTextAttributes(m_legend->textAttributes(), foreground, font().pointSizeF() - 2);
+        QFont legendFont(QStringLiteral("monospace"));
+        legendFont.setStyleHint(QFont::TypeWriter);
+        textAttr.setFont(legendFont);
+
+        m_legend->setBackgroundAttributes(bkgAtt);
+        m_legend->setTextAttributes(textAttr);
+    }
+
+    {
+        m_totalPlotter = new Plotter(this);
+        m_totalPlotter->setAntiAliasing(true);
         auto totalProxy = new ChartProxy(true, this);
         totalProxy->setSourceModel(model);
-        totalPlotter->setModel(totalProxy);
-        totalPlotter->setType(Plotter::Stacked);
+        m_totalPlotter->setModel(totalProxy);
+        m_totalPlotter->setType(Plotter::Stacked);
 
-        KColorScheme scheme(QPalette::Active, KColorScheme::Window);
-        const QPen foreground(scheme.foreground().color());
-        auto fixupTextAttributes = [&foreground](KChart::TextAttributes attributes, float pointSize) {
-            attributes.setPen(foreground);
-            auto fontSize = attributes.fontSize();
-            fontSize.setAbsoluteValue(pointSize);
-            attributes.setFontSize(fontSize);
-            return attributes;
-        };
-
-        m_bottomAxis = new TimeAxis(totalPlotter);
-        const auto axisTextAttributes = fixupTextAttributes(m_bottomAxis->textAttributes(), font().pointSizeF() - 2);
+        m_bottomAxis = new TimeAxis(m_totalPlotter);
+        const auto axisTextAttributes =
+            fixupTextAttributes(m_bottomAxis->textAttributes(), foreground, font().pointSizeF() - 2);
         m_bottomAxis->setTextAttributes(axisTextAttributes);
-        const auto axisTitleTextAttributes =
-            fixupTextAttributes(m_bottomAxis->titleTextAttributes(), font().pointSizeF() + (minimalMode ? (-2) : (+2)));
+        const auto axisTitleTextAttributes = fixupTextAttributes(m_bottomAxis->titleTextAttributes(), foreground,
+                                                                 font().pointSizeF() + (minimalMode ? (-2) : (+2)));
         m_bottomAxis->setTitleTextAttributes(axisTitleTextAttributes);
         m_bottomAxis->setPosition(CartesianAxis::Bottom);
-        totalPlotter->addAxis(m_bottomAxis);
+        m_totalPlotter->addAxis(m_bottomAxis);
 
         m_rightAxis = model->type() == ChartModel::Allocations || model->type() == ChartModel::Temporary
-            ? new CartesianAxis(totalPlotter)
-            : new SizeAxis(totalPlotter);
+            ? new CartesianAxis(m_totalPlotter)
+            : new SizeAxis(m_totalPlotter);
         m_rightAxis->setTextAttributes(axisTextAttributes);
         m_rightAxis->setTitleTextAttributes(axisTitleTextAttributes);
         m_rightAxis->setPosition(CartesianAxis::Right);
-        totalPlotter->addAxis(m_rightAxis);
+        m_totalPlotter->addAxis(m_rightAxis);
 
-        coordinatePlane->addDiagram(totalPlotter);
+        coordinatePlane->addDiagram(m_totalPlotter);
+
+        m_legend->addDiagram(m_totalPlotter);
     }
 
     {
-        auto plotter = new Plotter(this);
-        plotter->setAntiAliasing(true);
-        plotter->setType(Plotter::Stacked);
+        m_detailedPlotter = new Plotter(this);
+        m_detailedPlotter->setAntiAliasing(true);
+        m_detailedPlotter->setType(Plotter::Stacked);
 
         auto proxy = new ChartProxy(false, this);
         proxy->setSourceModel(model);
-        plotter->setModel(proxy);
-        coordinatePlane->addDiagram(plotter);
+        m_detailedPlotter->setModel(proxy);
+        coordinatePlane->addDiagram(m_detailedPlotter);
+
+        m_legend->addDiagram(m_detailedPlotter);
     }
 
+    m_legend->hide();
+
+    // If the dataset has 10 entries, one is for the total plot and the
+    // remaining ones are for the detailed plot. We want to only change
+    // the number of detailed plots, so we have to correct it.
+    int maximumDatasetCount = m_model->maximumDatasetCount();
+    m_stackedDiagrams->setValue(maximumDatasetCount - 1);
+
     updateToolTip();
     updateAxesTitle();
 }
 
+void ChartWidget::saveAs()
+{
+    const auto saveFilename =
+        QFileDialog::getSaveFileName(this, i18n("Save %1", windowTitle()), QString(),
+                                     i18n("Raster Image (*.png *.jpg *.tiff);;Vector Image (*.svg)"));
+
+    if (!saveFilename.isEmpty()) {
+        if (QFileInfo(saveFilename).suffix() == QLatin1String("svg")) {
+            // vector graphic format
+            QSvgGenerator generator;
+            generator.setFileName(saveFilename);
+            generator.setSize(m_chart->size());
+            generator.setViewBox(m_chart->rect());
+
+            QPainter painter;
+            painter.begin(&generator);
+            m_chart->paint(&painter, m_chart->rect());
+            painter.end();
+        } else if (!m_chart->grab().save(saveFilename)) {
+            // other format
+            KMessageBox::error(this, i18n("Failed to save the image to %1", saveFilename));
+        }
+    }
+}
+
 void ChartWidget::updateToolTip()
 {
     if (!m_model)
@@ -310,8 +462,9 @@ void ChartWidget::updateAxesTitle()
     if (!m_model)
         return;
 
-    m_bottomAxis->setTitleText(m_model->headerData(0).toString());
-    m_rightAxis->setTitleText(m_model->headerData(1).toString());
+    // m_bottomAxis is always time, so we can just write it here instead of in headerData().
+    m_bottomAxis->setTitleText(i18n("Elapsed Time"));
+    m_rightAxis->setTitleText(m_model->typeString());
 
     if (m_summaryData.filterParameters.isFilteredByTime(m_summaryData.totalTime)) {
         m_bottomAxis->setTitleText(
@@ -371,7 +524,7 @@ bool ChartWidget::eventFilter(QObject* watched, QEvent* event)
 
     if (auto* mouseEvent = dynamic_cast<QMouseEvent*>(event)) {
         if (mouseEvent->button() == Qt::LeftButton || mouseEvent->buttons() == Qt::LeftButton) {
-            const auto time = mapPosToTime(mouseEvent->localPos());
+            const auto time = mapPosToTime(localPos(mouseEvent));
 
             auto selection = m_selection;
             selection.end = time;
@@ -385,10 +538,10 @@ bool ChartWidget::eventFilter(QObject* watched, QEvent* event)
             }
 
             setSelection(selection);
-            QToolTip::showText(mouseEvent->globalPos(), toolTip(), this);
+            QToolTip::showText(globalPos(mouseEvent), toolTip(), this);
             return true;
         } else if (event->type() == QEvent::MouseMove && !mouseEvent->buttons()) {
-            updateStatusTip(mapPosToTime(mouseEvent->localPos()));
+            updateStatusTip(mapPosToTime(localPos(mouseEvent)));
         }
     } else if (event->type() == QEvent::Paint && !m_cachedChart.isNull()) {
         // use the cached chart while interacting with the rubber band
@@ -436,3 +589,5 @@ void ChartWidget::updateStatusTip(qint64 time)
 }
 
 #include "chartwidget.moc"
+
+#include "moc_chartwidget.cpp"
