@@ -14,17 +14,19 @@
 #include <KLocalizedString>
 #include <KShell>
 #include <KStandardAction>
+#include <kio_version.h>
 
 #include <QAction>
+#include <QActionGroup>
 #include <QClipboard>
 #include <QDebug>
 #include <QDesktopServices>
 #include <QFileDialog>
+#include <QInputDialog>
 #include <QMenu>
+#include <QProcess>
 #include <QShortcut>
 #include <QStatusBar>
-#include <QProcess>
-#include <QInputDialog>
 
 #include "analyze/suppressions.h"
 
@@ -65,32 +67,53 @@ const char IDE[] = "IDE";
 }
 }
 
+enum IDE
+{
+    KDevelop,
+    Kate,
+    KWrite,
+    GEdit,
+    GVim,
+    QtCreator,
+    LAST_IDE
+};
 struct IdeSettings
 {
-    const char* const app;
-    const char* const args;
-    const char* const name;
+    QString app;
+    QString args;
+    QString name;
+
+    bool isAppAvailable() const
+    {
+        return !QStandardPaths::findExecutable(app).isEmpty();
+    }
 };
 
-static const IdeSettings ideSettings[] = {
-    {"kdevelop", "%f:%l:%c", I18N_NOOP("KDevelop")},
-    {"kate", "%f --line %l --column %c", I18N_NOOP("Kate")},
-    {"kwrite", "%f --line %l --column %c", I18N_NOOP("KWrite")},
-    {"gedit", "%f +%l:%c", I18N_NOOP("gedit")},
-    {"gvim", "%f +%l", I18N_NOOP("gvim")},
-    {"qtcreator", "-client %f:%l", I18N_NOOP("Qt Creator")}
-};
-static const int ideSettingsSize = sizeof(ideSettings) / sizeof(IdeSettings);
-
-bool isAppAvailable(const char* app)
+IdeSettings ideSettings(IDE ide)
 {
-    return !QStandardPaths::findExecutable(QString::fromUtf8(app)).isEmpty();
-}
+    switch (ide) {
+    case KDevelop:
+        return {QStringLiteral("kdevelop"), QStringLiteral("%f:%l:%c"), MainWindow::tr("KDevelop")};
+    case Kate:
+        return {QStringLiteral("kate"), QStringLiteral("%f --line %l --column %c"), MainWindow::tr("Kate")};
+    case KWrite:
+        return {QStringLiteral("kwrite"), QStringLiteral("%f --line %l --column %c"), MainWindow::tr("KWrite")};
+    case GEdit:
+        return {QStringLiteral("gedit"), QStringLiteral("%f +%l:%c"), MainWindow::tr("gedit")};
+    case GVim:
+        return {QStringLiteral("gvim"), QStringLiteral("%f +%l"), MainWindow::tr("gvim")};
+    case QtCreator:
+        return {QStringLiteral("qtcreator"), QStringLiteral("-client %f:%l"), MainWindow::tr("Qt Creator")};
+    case LAST_IDE:
+        break;
+    };
+    Q_UNREACHABLE();
+};
 
 int firstAvailableIde()
 {
-    for (int i = 0; i < ideSettingsSize; ++i) {
-        if (isAppAvailable(ideSettings[i].app)) {
+    for (int i = 0; i < LAST_IDE; ++i) {
+        if (ideSettings(static_cast<IDE>(i)).isAppAvailable()) {
             return i;
         }
     }
@@ -540,6 +563,16 @@ MainWindow::MainWindow(QWidget* parent)
         return false;
     };
 
+    const QString heaptrackFileFilter = QStringLiteral("heaptrack.*.*.gz heaptrack.*.*.zst");
+#if KIO_VERSION >= QT_VERSION_CHECK(5, 108, 0)
+    const QStringList heaptrackFileFilters = {heaptrackFileFilter};
+    m_ui->openFile->setNameFilters(heaptrackFileFilters);
+    m_ui->compareTo->setNameFilters(heaptrackFileFilters);
+#else
+    m_ui->openFile->setFilter(heaptrackFileFilter);
+    m_ui->compareTo->setFilter(heaptrackFileFilter);
+#endif
+
     auto validateInput = [this, validateInputFile]() {
         m_ui->messages->hide();
         m_ui->buttonBox->setEnabled(validateInputFile(m_ui->openFile->url().toLocalFile(), false)
@@ -746,7 +779,6 @@ void MainWindow::setupStacks()
     m_ui->stacksDock->setVisible(false);
 }
 
-
 void MainWindow::setupCodeNavigationMenu()
 {
     // Code Navigation
@@ -759,10 +791,11 @@ void MainWindow::setupCodeNavigationMenu()
     const auto settings = m_config->group(Config::Groups::CodeNavigation);
     const auto currentIdx = settings.readEntry(Config::Entries::IDE, firstAvailableIde());
 
-    for (int i = 0; i < ideSettingsSize; ++i) {
+    for (int i = 0; i < LAST_IDE; ++i) {
         auto action = new QAction(menu);
-        action->setText(i18n(ideSettings[i].name));
-        auto icon = QIcon::fromTheme(QString::fromUtf8(ideSettings[i].app));
+        auto ide = ideSettings(static_cast<IDE>(i));
+        action->setText(ide.name);
+        auto icon = QIcon::fromTheme(ide.app);
         if (icon.isNull()) {
             icon = QIcon::fromTheme(QStringLiteral("application-x-executable"));
         }
@@ -770,9 +803,7 @@ void MainWindow::setupCodeNavigationMenu()
         action->setCheckable(true);
         action->setChecked(currentIdx == i);
         action->setData(i);
-#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0) // It's not worth it to reimplement missing findExecutable for Qt4.
-        action->setEnabled(isAppAvailable(ideSettings[i].app));
-#endif
+        action->setEnabled(ide.isAppAvailable());
         group->addAction(action);
         menu->addAction(action);
     }
@@ -813,7 +844,7 @@ void MainWindow::setCodeNavigationIDE(QAction* action)
         const auto customCmd =
             QInputDialog::getText(this, i18n("Custom Code Navigation"),
                                   i18n("Specify command to use for code navigation, '%f' will be replaced by the file "
-                                     "name, '%l' by the line number and '%c' by the column number."),
+                                       "name, '%l' by the line number and '%c' by the column number."),
                                   QLineEdit::Normal, settings.readEntry(Config::Entries::CustomCommand));
         if (!customCmd.isEmpty()) {
             settings.writeEntry(Config::Entries::CustomCommand, customCmd);
@@ -832,8 +863,9 @@ void MainWindow::navigateToCode(const QString& filePath, int lineNumber, int col
     const auto ideIdx = settings.readEntry(Config::Entries::IDE, firstAvailableIde());
 
     QString command;
-    if (ideIdx >= 0 && ideIdx < ideSettingsSize) {
-        command = QString::fromUtf8(ideSettings[ideIdx].app) + QLatin1Char(' ') + QString::fromUtf8(ideSettings[ideIdx].args);
+    if (ideIdx >= 0 && ideIdx < LAST_IDE) {
+        auto ide = ideSettings(static_cast<IDE>(ideIdx));
+        command = ide.app + QLatin1Char(' ') + ide.args;
     } else if (ideIdx == -1) {
         command = settings.readEntry(Config::Entries::CustomCommand);
     }
@@ -864,3 +896,5 @@ void MainWindow::setSuppressions(std::vector<std::string> suppressions)
 {
     m_lastFilterParameters.suppressions = std::move(suppressions);
 }
+
+#include "moc_mainwindow.cpp"
