@@ -23,11 +23,11 @@
 #include <syscall.h>
 #endif
 #ifdef __FreeBSD__
-#include <pthread_np.h>
-#include <sys/types.h>
-#include <sys/sysctl.h>
-#include <sys/user.h>
 #include <libutil.h>
+#include <pthread_np.h>
+#include <sys/sysctl.h>
+#include <sys/types.h>
+#include <sys/user.h>
 #endif
 #include <sys/file.h>
 
@@ -43,6 +43,7 @@
 #include "util/config.h"
 #include "util/libunwind_config.h"
 #include "util/linewriter.h"
+#include "util/macroutils.h"
 
 extern "C" {
 // see upstream "documentation" at:
@@ -170,13 +171,13 @@ void printBacktrace()
  * Set to true in an atexit handler. In such conditions, the stop callback
  * will not be called.
  */
-atomic<bool> s_atexit{false};
+atomic<bool> s_atexit {false};
 
 /**
  * Set to true in heaptrack_stop, when s_atexit was not yet set. In such conditions,
  * we always fully unload and cleanup behind ourselves
  */
-atomic<bool> s_forceCleanup{false};
+atomic<bool> s_forceCleanup {false};
 
 // based on: https://stackoverflow.com/a/24315631/35250
 void replaceAll(string& str, const string& search, const string& replace)
@@ -256,16 +257,28 @@ int createFile(const char* fileName)
 class HeapTrack
 {
 public:
-    HeapTrack(const RecursionGuard& /*recursionGuard*/)
+    template <typename Op>
+    static bool op(const RecursionGuard& /*recursionGuard*/, const Op& op)
     {
         debugLog<VeryVerboseOutput>("%s", "acquiring lock");
-        s_lock.lock();
+
+        auto locked = tryLock([]() { return s_forceCleanup.load(); });
+        if (!locked) {
+            return false;
+        }
+
         debugLog<VeryVerboseOutput>("%s", "lock acquired");
+
+        HeapTrack heaptrack(locked);
+        op(heaptrack);
+
+        return true;
     }
 
     ~HeapTrack()
     {
         debugLog<VeryVerboseOutput>("%s", "releasing lock");
+
         s_lock.unlock();
     }
 
@@ -445,7 +458,7 @@ public:
 #ifdef __linux__
         ssize_t size = readlink("/proc/self/exe", buf, BUF_SIZE);
 #elif defined(__FreeBSD__)
-        int mib[4] = { CTL_KERN, KERN_PROC, KERN_PROC_PATHNAME, -1 };
+        int mib[4] = {CTL_KERN, KERN_PROC, KERN_PROC_PATHNAME, -1};
         size_t size = BUF_SIZE;
         sysctl(mib, 4, buf, &size, NULL, 0);
 #endif
@@ -467,7 +480,7 @@ public:
         int bytesRead = read(fd, buf, BUF_SIZE);
         close(fd);
 #elif defined(__FreeBSD__)
-        int mib[4] = { CTL_KERN, KERN_PROC, KERN_PROC_ARGS, getpid() };
+        int mib[4] = {CTL_KERN, KERN_PROC, KERN_PROC_ARGS, getpid()};
         size_t bytesRead = BUF_SIZE;
         sysctl(mib, 4, buf, &bytesRead, NULL, 0);
 #endif
@@ -631,22 +644,47 @@ private:
         shutdown();
     }
 
-    struct LockCheckFailed{};
+    class LockStatus
+    {
+    public:
+        LockStatus(bool isLocked)
+            : isLocked(isLocked)
+        {
+        }
+        explicit operator bool() const
+        {
+            return isLocked;
+        }
+
+    private:
+        bool isLocked = false;
+    };
 
     /**
      * To prevent deadlocks on shutdown, we try to lock from the timer thread
-     * and throw an LockCheckFailed exception otherwise.
+     *
+     * TODO: c++17 return std::optional<HeapTrack>
      */
-    template <typename AdditionalLockCheck>
-    HeapTrack(AdditionalLockCheck lockCheck)
+    template <typename StopLockCheck>
+    static LockStatus tryLock(StopLockCheck stopLockCheck)
     {
         debugLog<VeryVerboseOutput>("%s", "trying to acquire lock");
         while (!s_lock.try_lock()) {
-            if (!lockCheck())
-                throw LockCheckFailed();
+            if (stopLockCheck()) {
+                return false;
+            }
             this_thread::sleep_for(chrono::microseconds(1));
         }
         debugLog<VeryVerboseOutput>("%s", "lock acquired");
+        return true;
+    }
+
+    /**
+     * Create locked HeapTrack instance after a successful call to tryLock
+     */
+    HeapTrack(POTENTIALLY_UNUSED LockStatus lockStatus)
+    {
+        assert(lockStatus);
     }
 
     struct LockedData
@@ -682,30 +720,19 @@ private:
                 RecursionGuard::isActive = true;
                 debugLog<MinimalOutput>("%s", "timer thread started");
 
-                // HACK: throw the exception once and directly catch it
-                //       without this, tst_inject reproducibly calls
-                //       std::terminate instead of catching the exception
-                //       in the loop below
-                //       I suspect it's some strange side-effect of heaptrack
-                //       intercepting the memory allocations that happen when
-                //       an exception is thrown?
-                try {
-                    throw LockCheckFailed {};
-                } catch (LockCheckFailed) {
-                }
-
                 // now loop and repeatedly print the timestamp and RSS usage to the data stream
                 while (!stopTimerThread) {
                     // TODO: make interval customizable
                     this_thread::sleep_for(chrono::milliseconds(10));
 
-                    try {
-                        HeapTrack heaptrack([&] { return !stopTimerThread.load(); });
-                        heaptrack.writeTimestamp();
-                        heaptrack.writeRSS();
-                    } catch (LockCheckFailed) {
+                    const auto locked = tryLock([&] { return stopTimerThread.load(); });
+                    if (!locked) {
                         break;
                     }
+
+                    HeapTrack heaptrack(locked);
+                    heaptrack.writeTimestamp();
+                    heaptrack.writeRSS();
                 }
             });
 
@@ -754,7 +781,7 @@ private:
 
         TraceTree traceTree;
 
-        atomic<bool> stopTimerThread{false};
+        atomic<bool> stopTimerThread {false};
         std::thread timerThread;
 
         heaptrack_callback_t stopCallback = nullptr;
@@ -772,8 +799,8 @@ private:
 };
 
 std::mutex HeapTrack::s_lock;
-HeapTrack::LockedData* HeapTrack::s_data{nullptr};
-std::atomic<bool> HeapTrack::s_paused{false};
+HeapTrack::LockedData* HeapTrack::s_data {nullptr};
+std::atomic<bool> HeapTrack::s_paused {false};
 }
 
 static void heaptrack_realloc_impl(void* ptr_in, size_t size, void* ptr_out)
@@ -786,11 +813,12 @@ static void heaptrack_realloc_impl(void* ptr_in, size_t size, void* ptr_out)
         Trace trace;
         trace.fill(2 + HEAPTRACK_DEBUG_BUILD * 3);
 
-        HeapTrack heaptrack(guard);
-        if (ptr_in) {
-            heaptrack.handleFree(ptr_in);
-        }
-        heaptrack.handleMalloc(ptr_out, size, trace);
+        HeapTrack::op(guard, [&](HeapTrack& heaptrack) {
+            if (ptr_in) {
+                heaptrack.handleFree(ptr_in);
+            }
+            heaptrack.handleMalloc(ptr_out, size, trace);
+        });
     }
 }
 
@@ -802,11 +830,14 @@ void heaptrack_init(const char* outputFileName, heaptrack_callback_t initBeforeC
     RecursionGuard guard;
     // initialize
     startTime();
+    s_forceCleanup.store(false);
 
     debugLog<MinimalOutput>("heaptrack_init(%s)", outputFileName);
 
-    HeapTrack heaptrack(guard);
-    heaptrack.initialize(outputFileName, initBeforeCallback, initAfterCallback, stopCallback);
+    POTENTIALLY_UNUSED auto ret = HeapTrack::op(guard, [&](HeapTrack& heaptrack) {
+        heaptrack.initialize(outputFileName, initBeforeCallback, initAfterCallback, stopCallback);
+    });
+    assert(ret);
 }
 
 void heaptrack_stop()
@@ -815,13 +846,15 @@ void heaptrack_stop()
 
     debugLog<MinimalOutput>("%s", "heaptrack_stop()");
 
-    HeapTrack heaptrack(guard);
+    assert(!s_forceCleanup);
+    POTENTIALLY_UNUSED auto ret = HeapTrack::op(guard, [&](HeapTrack& heaptrack) {
+        if (!s_atexit) {
+            s_forceCleanup.store(true);
+        }
 
-    if (!s_atexit) {
-        s_forceCleanup.store(true);
-    }
-
-    heaptrack.shutdown();
+        heaptrack.shutdown();
+    });
+    assert(ret);
 }
 
 void heaptrack_pause()
@@ -844,8 +877,7 @@ void heaptrack_malloc(void* ptr, size_t size)
         Trace trace;
         trace.fill(2 + HEAPTRACK_DEBUG_BUILD * 2);
 
-        HeapTrack heaptrack(guard);
-        heaptrack.handleMalloc(ptr, size, trace);
+        HeapTrack::op(guard, [&](HeapTrack& heaptrack) { heaptrack.handleMalloc(ptr, size, trace); });
     }
 }
 
@@ -856,8 +888,7 @@ void heaptrack_free(void* ptr)
 
         debugLog<VeryVerboseOutput>("heaptrack_free(%p)", ptr);
 
-        HeapTrack heaptrack(guard);
-        heaptrack.handleFree(ptr);
+        HeapTrack::op(guard, [&](HeapTrack& heaptrack) { heaptrack.handleFree(ptr); });
     }
 }
 
@@ -877,8 +908,7 @@ void heaptrack_invalidate_module_cache()
 
     debugLog<VerboseOutput>("%s", "heaptrack_invalidate_module_cache()");
 
-    HeapTrack heaptrack(guard);
-    heaptrack.invalidateModuleCache();
+    HeapTrack::op(guard, [&](HeapTrack& heaptrack) { heaptrack.invalidateModuleCache(); });
 }
 
 void heaptrack_warning(heaptrack_warning_callback_t callback)
