@@ -4,7 +4,8 @@
     SPDX-License-Identifier: LGPL-2.1-or-later
 */
 
-#include "3rdparty/catch.hpp"
+#define DOCTEST_CONFIG_IMPLEMENT
+#include "3rdparty/doctest.h"
 
 #include "analyze/suppressions.h"
 #include "locationdata.h"
@@ -12,6 +13,8 @@
 #include "treemodel.h"
 #include "tst_config.h" // for SRC_DIR
 
+#include <KLocalizedString>
+
 #include <QDebug>
 #include <QSignalSpy>
 
@@ -37,13 +40,13 @@ struct TestParser
     ~TestParser()
     {
         if (spyFinished.isEmpty())
-            REQUIRE(spyFinished.wait(10000));
+            REQUIRE(spyFinished.wait(20000));
     }
 
     CallerCalleeResults awaitCallerCallee()
     {
         if (spyCCD.isEmpty())
-            REQUIRE(spyCCD.wait());
+            REQUIRE(spyCCD.wait(20000));
 
         auto ccr = spyCCD.at(0).at(0).value<CallerCalleeResults>();
         REQUIRE(ccr.resultData);
@@ -54,7 +57,7 @@ struct TestParser
     TreeData awaitBottomUp()
     {
         if (spyBottomUp.isEmpty())
-            REQUIRE(spyBottomUp.wait(10000));
+            REQUIRE(spyBottomUp.wait(20000));
 
         auto bottomUpData = spyBottomUp.at(0).at(0).value<TreeData>();
         REQUIRE(bottomUpData.resultData);
@@ -73,7 +76,7 @@ struct TestParser
     TreeData awaitTopDown()
     {
         if (spyTopDown.isEmpty())
-            REQUIRE(spyTopDown.wait(10000));
+            REQUIRE(spyTopDown.wait(20000));
 
         auto topDownData = spyTopDown.at(0).at(0).value<TreeData>();
         REQUIRE(topDownData.resultData);
@@ -92,7 +95,7 @@ struct TestParser
     SummaryData awaitSummary()
     {
         if (spySummary.isEmpty())
-            REQUIRE(spySummary.wait(10000));
+            REQUIRE(spySummary.wait(20000));
 
         return spySummary.at(0).at(0).value<SummaryData>();
     }
@@ -135,7 +138,7 @@ private:
     QSignalSpy spyFinished;
 };
 
-TEST_CASE ("heaptrack.david.18594.gz", "[parser]") {
+TEST_CASE ("heaptrack.david.18594.gz") {
     TestParser parser;
 
     FilterParameters params;
@@ -202,7 +205,7 @@ TEST_CASE ("heaptrack.david.18594.gz", "[parser]") {
     REQUIRE(summary.fromAttached == false);
 }
 
-TEST_CASE ("heaptrack.embedded_lsan_suppressions.84207.zst", "[parser]") {
+TEST_CASE ("heaptrack.embedded_lsan_suppressions.84207.zst") {
     TestParser parser;
 
     parser.parser.parse(SRC_DIR "/heaptrack.embedded_lsan_suppressions.84207.zst", QString(), {});
@@ -217,7 +220,7 @@ TEST_CASE ("heaptrack.embedded_lsan_suppressions.84207.zst", "[parser]") {
     REQUIRE(summary.totalSystemMemory == 33643876352);
 }
 
-TEST_CASE ("heaptrack.embedded_lsan_suppressions.84207.zst without suppressions", "[parser]") {
+TEST_CASE ("heaptrack.embedded_lsan_suppressions.84207.zst without suppressions") {
     TestParser parser;
 
     FilterParameters params;
@@ -231,7 +234,7 @@ TEST_CASE ("heaptrack.embedded_lsan_suppressions.84207.zst without suppressions"
     REQUIRE(summary.totalLeakedSuppressed == 0);
 }
 
-TEST_CASE ("heaptrack.heaptrack_gui.99454.zst", "[parser]") {
+TEST_CASE ("heaptrack.heaptrack_gui.99454.zst") {
     TestParser parser;
 
     FilterParameters params;
@@ -262,7 +265,7 @@ TEST_CASE ("heaptrack.heaptrack_gui.99454.zst", "[parser]") {
     CHECK(cost.selfCost.peak == 56152);
 }
 
-TEST_CASE ("heaptrack.heaptrack_gui.99529.zst", "[parser]") {
+TEST_CASE ("heaptrack.heaptrack_gui.99529.zst") {
     TestParser parser;
 
     FilterParameters params;
@@ -293,7 +296,7 @@ TEST_CASE ("heaptrack.heaptrack_gui.99529.zst", "[parser]") {
     CHECK(cost.selfCost.peak == 68952);
 }
 
-TEST_CASE ("heaptrack.heaptrack_gui.{99454,99529}.zst diff", "[parser]") {
+TEST_CASE ("heaptrack.heaptrack_gui.{99454,99529}.zst diff") {
     TestParser parser;
 
     parser.parser.parse(SRC_DIR "/heaptrack.heaptrack_gui.99529.zst", SRC_DIR "/heaptrack.heaptrack_gui.99454.zst", {});
@@ -326,7 +329,10 @@ int main(int argc, char** argv)
     QCoreApplication app(argc, argv);
 
     qRegisterMetaType<CallerCalleeResults>();
+    KLocalizedString::setApplicationDomain("heaptrack");
 
-    const int res = Catch::Session().run(argc, argv);
-    return (res < 0xff ? res : 0xff);
+    doctest::Context context;
+    context.applyCommandLine(argc, argv);
+
+    return context.run();
 }
