@@ -1,23 +1,12 @@
 /*
- * Copyright 2014-2017 Milian Wolff <mail@milianw.de>
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
- */
+    SPDX-FileCopyrightText: 2014-2017 Milian Wolff <mail@milianw.de>
+
+    SPDX-License-Identifier: LGPL-2.1-or-later
+*/
 
 #include <cstdio>
 #include <cstdlib>
+#include <unistd.h>
 
 #include "util/config.h"
 
@@ -100,8 +89,8 @@ int main()
     printf("valloc: %p\n", buf);
     free(buf);
 
-    posix_memalign(&buf, 16, 64);
-    printf("posix_memalign: %p\n", buf);
+    int ret = posix_memalign(&buf, 16, 64);
+    printf("posix_memalign: %d %p\n", ret, buf);
     free(buf);
 
     for (int i = 0; i < 10; ++i) {
